//=============================================================================
// NtSpdCln.c
// Program reads UDP packets sending by NtSpdCln
//=============================================================================
#define INCL_WIN
#define INCL_DOSPROCESS

#include <os2.h>
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <ctype.h>
#include <netdb.h>
#include <types.h>
#include <sys\socket.h>
#include <net\if.h>
#include <arpa\inet.h>
#include <unistd.h>
#include <process.h>
#include "NetSpeed.h"

//-----------------------------------------------------------------------------
// Prototypes
//-----------------------------------------------------------------------------
void DoRun(void *);
BOOL resolve(char *, struct sockaddr_in *);

//-----------------------------------------------------------------------------
// Global Variablies
//-----------------------------------------------------------------------------
unsigned __MaxThreads = { MAX_THREADS + 1 };
struct sockaddr_in myaddr_udp = { 0 };
UDPPKT INITPKT = { 0 };

//=============================================================================
// Main procedure
//=============================================================================
void main(int argc, char *argv[])
{
TID tid[MAX_THREADS] = { 0 };
int udpsock, i;
unsigned short Port;
char drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];

// Init the variables
  strcpy(INITPKT.Msg, FirstMsgTxt);
  _splitpath(argv[0], drive, dir, fname, ext); 

// Check input parameters
  if ( argc < 3 )
    {
    printf("%s - Network Speed Measuring Client\n\n", fname);
    printf("Syntax:\n");
    printf("  %s <Address> <Port> [Threads] [Packets]\n\n", fname);
    printf("Here:\n");
    printf("  Address - Host name or IP address of Server.\n");
    printf("  Port    - Server's UDP port number.\n");
    printf("  Threads - Number of threads sending UDP packets.\n");
    printf("            Valid range is %d to %d (by default is %d).\n",
           MIN_THREADS, MAX_THREADS, DEF_THREADS);
    printf("  Packets - Each thread sends number of packets.\n");
    printf("            Valid range is %d to %d (by default is %d).\n",
           MIN_PACKETS, MAX_PACKETS, DEF_PACKETS);

    return;
    }

  if ( !resolve(argv[1], &myaddr_udp) )
    {
    printf("Unknown IP address %s\n", argv[1]);
    return;
    }

  Port = atoi(argv[2]);

  if ( argc > 3 ) INITPKT.Threads = atoi(argv[3]);
  if ( (INITPKT.Threads < MIN_THREADS) || (INITPKT.Threads > MAX_THREADS) )
    INITPKT.Threads = DEF_THREADS;

  if ( argc > 4 ) INITPKT.Packets = atoi(argv[4]);
  if ( (INITPKT.Packets < MIN_PACKETS) || (INITPKT.Packets > MAX_PACKETS) )
    INITPKT.Packets = DEF_PACKETS;

  printf("IP address of Server - %s\n",
         inet_ntoa(*(struct in_addr *)&myaddr_udp.sin_addr));
  printf("UDP port number      - %d\n", Port);
  printf("Number of threads    - %d\n", INITPKT.Threads);
  printf("Number of packets    - %d\n", INITPKT.Packets);

// Send the first packet
  udpsock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  myaddr_udp.sin_family = AF_INET;
  myaddr_udp.sin_port = htons(Port);
  sendto(udpsock, (char *)&INITPKT, sizeof(UDPPKT), 0,
         (struct sockaddr *)&myaddr_udp, sizeof(struct sockaddr_in));
  soclose(udpsock);

// Create Threads
  for (i=0; i<INITPKT.Threads; i++)
    tid[i] = _beginthread(DoRun, NULL, 65536, (void *)i);

// Wait until all the threads finish
  for (i=0; i<INITPKT.Threads; i++)
    DosWaitThread (&tid[i], DCWW_WAIT);
}


//=============================================================================
// DoRun - send packets to Server
//=============================================================================
void DoRun(void *parm01)
{
UDPPKT TESTPKT = { 0 };
int sock;
struct sockaddr_in myaddr;

// Init the test packet
  strcpy(TESTPKT.Msg, TestMsgTxt);
  TESTPKT.Threads = (int)parm01;

// Send the test packets
  sock = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  memcpy((char *)&myaddr, (char *)&myaddr_udp, sizeof(struct sockaddr_in));

  for (TESTPKT.Packets=0; TESTPKT.Packets<INITPKT.Packets; TESTPKT.Packets++)
    sendto(sock, (char *)&TESTPKT, sizeof(UDPPKT), 0,
        (struct sockaddr *)&myaddr, sizeof(struct sockaddr_in));

  soclose(sock);
}

//=============================================================================
// resolve - build IP address
//=============================================================================
BOOL resolve(char *address, struct sockaddr_in *myaddr)
{
struct hostent *host;

  if ( (myaddr->sin_addr.s_addr = inet_addr(address)) == INADDR_NONE )
    {
    if ( (host = gethostbyname(address)) == NULL ) return FALSE;
    memcpy((char *)&(myaddr->sin_addr), (int *)host->h_addr, host->h_length);
    }

  return TRUE;
}
